Attribute VB_Name = "FileFolderUtility"
'*********************************************************************
'Titolo: FileFolderUtility.bas
'Autore: Giorgio Bernardi
'E-Mail: BGSoftware@hotmail.com
'Data  : Novembre 2000
'Descrizione: Questo Modulo serve per ottenere numerose informazioni
'           su Files o Cartelle.
'Info:  Potete usare e modificare a piacimento questo codice
'*********************************************************************
Option Explicit
'**************************************************************************
'Dichiarazione Funzioni Api, Tipi e Strutture
'**************************************************************************
Declare Function FindFirstFile Lib "kernel32" Alias "FindFirstFileA" (ByVal lpFileName As String, lpFindFileData As WIN32_FIND_DATA) As Long
Declare Function FindClose Lib "kernel32" (ByVal hFindFile As Long) As Long
Declare Function FileTimeToSystemTime Lib "kernel32" (lpFileTime As FILETIME, lpSystemTime As SYSTEMTIME) As Long
Declare Function FileTimeToLocalFileTime Lib "kernel32" (lpFileTime As FILETIME, lpLocalFileTime As FILETIME) As Long

Declare Function GetVolumeInformation Lib "kernel32" Alias "GetVolumeInformationA" (ByVal lpRootPathName As String, ByVal lpVolumeNameBuffer As String, ByVal nVolumeNameSize As Long, lpVolumeSerialNumber As Long, lpMaximumComponentLength As Long, lpFileSystemFlags As Long, ByVal lpFileSystemNameBuffer As String, ByVal nFileSystemNameSize As Long) As Long

Private Type SYSTEMTIME
  wYear As Integer
  wMonth As Integer
  wDayOfWeek As Integer
  wDay As Integer
  wHour As Integer
  wMinute As Integer
  wSecond As Integer
  wMilliseconds As Integer
End Type

Type FILETIME
  LowDateTime          As Long
  HighDateTime         As Long
End Type

Type WIN32_FIND_DATA
  dwFileAttributes     As Long
  ftCreationTime       As FILETIME
  ftLastAccessTime     As FILETIME
  ftLastWriteTime      As FILETIME
  nFileSizeHigh        As Long
  nFileSizeLow         As Long
  dwReserved0          As Long
  dwReserved1          As Long
  cFileName            As String * 260  'MUST be set to 260
  cAlternate           As String * 14
End Type

'**************************************************************************
'Dichiarazione Tipi e Strutture Inventate da me per semplificare alcune operazioni
'**************************************************************************
'Gli attributi di un file eseguibile (si leggono con un click con
'il tasto destro del mouse sul file e scegliendo nel menu "Propriet"
'andando poi nella sezione "Versione"). N.B. Ogni eseguibile pu avere
'o non avere una o pi fra queste informazioni!
Public Enum exeAttributes
    ProductName
    CompanyName
    FileVersion
    FileVersionMayor
    FileVersionMinor
    FileVersionRevision
    ProductVersion
    ProductVersionMayor
    ProductVersionMinor
    ProductVersionRevision
    Copyrights
    Description
    Comments
    OriginalFileName
    InternalName
End Enum

'Gli attributi possibili di un file (o cartella)
Public Enum FileAttributes
    ReadOnlyAtt = 0
    HiddenAtt = 1
    SystemAtt = 2
    DirectoryAtt = 3
    ArchiveAtt = 4
    NormalAtt = 5
    TemporaryAtt = 6
    CompressedAtt = 7
End Enum

'Struttura che contiene le tre date di un file
Public Type FileDateTime
    lastAccessed As String
    lastModified As String
    created As String
End Type

'**************************************************************
'***********    LISTA DELLE FUNZIONI PUBBLICHE    *************
'**************************************************************

Public Function getFileLenght(fileName As String, Optional UseApi As Boolean = False) As Long
'Restituisce la "grandezza" di un file in Byte. Utilizza FileLen
'contenuta in VBA.System, oppure direttamente una Api
If UseApi Then
    Dim fileData As WIN32_FIND_DATA
    fileData = Findfile(fileName)
    If fileData.nFileSizeHigh = 0 Then
        getFileLenght = fileData.nFileSizeLow
    Else
        getFileLenght = fileData.nFileSizeHigh
    End If
Else
    getFileLenght = FileLen(fileName)
End If
End Function

Public Function getFileDateTime(fileName As String) As FileDateTime
'Restituisce un Tipo FileDateTime che contiene le "tre date" di un file:
'La data di creazione, di ultimo accesso e di ultima modifica.

Dim ftime As SYSTEMTIME
Dim localTime As FILETIME
Dim fileData As WIN32_FIND_DATA

fileData = Findfile(fileName)        'Ottengo le informazioni

'Data di Creazione
If fileData.ftCreationTime.HighDateTime = 0 And fileData.ftCreationTime.LowDateTime = 0 Then
    getFileDateTime.created = "(sconosciuto)"
Else
    Call FileTimeToLocalFileTime(fileData.ftCreationTime, localTime)    'Trasformazione nell'ora locale
    Call FileTimeToSystemTime(localTime, ftime)
    getFileDateTime.created = imponiDueCifre(ftime.wDay) & "/" & imponiDueCifre(ftime.wMonth) & "/" & imponiDueCifre(ftime.wYear) & " " & imponiDueCifre(ftime.wHour) & ":" & imponiDueCifre(ftime.wMinute) & ":" & imponiDueCifre(ftime.wSecond)
End If
'Ultima modifica
If fileData.ftLastWriteTime.HighDateTime = 0 And fileData.ftLastWriteTime.LowDateTime = 0 Then
    getFileDateTime.lastModified = "(sconosciuto)"
Else
    Call FileTimeToLocalFileTime(fileData.ftLastWriteTime, localTime)
    Call FileTimeToSystemTime(localTime, ftime)
    getFileDateTime.lastModified = imponiDueCifre(ftime.wDay) & "/" & imponiDueCifre(ftime.wMonth) & "/" & imponiDueCifre(ftime.wYear) & " " & imponiDueCifre(ftime.wHour) & ":" & imponiDueCifre(ftime.wMinute) & ":" & imponiDueCifre(ftime.wSecond)
End If

'Ultimo accesso
If fileData.ftLastAccessTime.HighDateTime = 0 And fileData.ftLastAccessTime.LowDateTime = 0 Then
    getFileDateTime.lastAccessed = "(sconosciuto)"
Else
    Call FileTimeToLocalFileTime(fileData.ftLastAccessTime, localTime)
    Call FileTimeToSystemTime(localTime, ftime)
    getFileDateTime.lastAccessed = imponiDueCifre(ftime.wDay) & "/" & imponiDueCifre(ftime.wMonth) & "/" & imponiDueCifre(ftime.wYear)
End If

End Function

Public Function getFileAttributes(fileName As String, Optional UseApi As Boolean = False) As Integer
'Restituisce un numero che identifica tutti gli attributi di un file
'Per sapere se un file ha o non ha un determinato attributo utilizzare
'la funzione Booleana isFileAttribute(..)
If UseApi Then
    Dim fileData As WIN32_FIND_DATA
    fileData = Findfile(fileName)        'Ottengo le informazioni
    getFileAttributes = fileData.dwFileAttributes
Else
    getFileAttributes = GetAttr(fileName)
End If
End Function

Public Function isFileAttribute(fileName As String, attributo As FileAttributes, Optional UseApi As Boolean = False) As Boolean
'Restituisce TRUE se il file di percorso fileName possiede l'attributo indicato
'altrimenti restituisce False
Dim myCheck As Long
Dim fileAtt As Long

Select Case attributo       'Deduco dal tipo FileAttributes il valore
    Case ReadOnlyAtt        'Esadecimale corrispondente
        myCheck = &H1
    Case HiddenAtt
        myCheck = &H2
    Case SystemAtt
        myCheck = &H4
    Case DirectoryAtt
        myCheck = &H10
    Case ArchiveAtt
        myCheck = &H20
    Case NormalAtt
        myCheck = &H80
    Case TemporaryAtt
        myCheck = &H100
    Case CompressedAtt
        myCheck = &H800
End Select

fileAtt = getFileAttributes(fileName, UseApi)  'Ottengo tutti gli attributi

'Ora controllo che l'operazione And fatta sugli attributi con l'attributo cercato,
' uguale all'attributo.
'(In binario ricordate che, per esempio, 1101 And 0100 restituisce
'0100 cio un 1 solo dove i due bit corrispondenti degli operatori
'sono uguali a 1).
If (fileAtt And myCheck) = myCheck Then isFileAttribute = True Else isFileAttribute = False

End Function

Public Function getFolderLenght(ByVal dirName As String, Optional RecurseFolder As Boolean = False, Optional UseApi As Boolean = False) As Currency
'Restituisce la "grandezza" di una cartella in Byte.
'Se recurseFolder = true calcola anche i byte delle sotto cartelle
'incluse in quella richiesta.

    Dim FileSize As Currency
    Dim NextF As Integer
    Dim Folders() As String
    ReDim Folders(0)            'Registra tutte le sottocartelle
    Dim fileName As String
    Dim t As Integer
    
    FileSize = 0                'Inizializzo la grandezza
    
    'Se la cartella  stata passata senza la barra la metto
    If Right(dirName, 1) <> "\" Then dirName = dirName & "\"
    
    'Richiedo il primo file
    '30 indica la somma di tutti i possibili attributi
    fileName = Dir$(dirName & "*.*", 30)
    Do While fileName <> ""         'Continuo fino a che Dir$ mi passa un file
        If fileName <> "." And fileName <> ".." Then    'A meno che non mi passi . o ..
            If isFileAttribute(dirName & fileName, DirectoryAtt, UseApi) Then ' una cartella
                If RecurseFolder Then Call AddToArray(Folders, fileName)  'Aggiungo la cartella all'array
            Else    'se  un file lo aggiungo al conteggio della grandezza
                FileSize = FileSize + getFileLenght(dirName & fileName, UseApi)
            End If
        End If
        fileName = Dir$          'Richiedo un altro file
    Loop
    
    For t = 1 To UBound(Folders)    'Ricorsione per le sottocartelle
        FileSize = FileSize + getFolderLenght(dirName & Folders(t) & "\", True, UseApi)
    Next t
    
    getFolderLenght = FileSize      'Restituisco la grandezza calcolata
End Function

Public Function getFileExtension(NomeFile As String) As String
'Restituisce l'estensione del file (per esempio "txt" ecc...
'in formato minuscolo
Dim tInt As Integer
Dim i As Integer
'Inizializzo la risposta nel caso in cui il file non abbia l'estensione
getFileExtension = ""

tInt = Len(NomeFile)
For i = tInt - 1 To 1 Step -1
    If Mid(NomeFile, i, 1) = "." Then getFileExtension = LCase(Right(NomeFile, tInt - i)): Exit For
Next i
End Function

Public Sub fileNamePath(Percorso As String, Optional ByRef fileName As String, Optional ByRef filePath As String)
'dato Percorso inserisce in fileName il nome del file, in filePath il percorso
'se Percorso  semplicemente il nome del file restituisce il nome del file
Dim tro As Long
Dim tro1 As Long
tro1 = 0
Do
    tro = tro1 + 1
    tro1 = InStr(tro, Percorso, "\")
Loop While (tro1 <> 0)

tro = tro - 1

fileName = Right(Percorso, Len(Percorso) - tro)
filePath = Left(Percorso, tro)
End Sub

Public Function getFileName(Percorso As String, Optional removeExtension As Boolean = False) As String
'Restituisce soltanto il nome del file dato un percorso
'Eventualmente restituisce il nome senza estensione
Dim temp As String
Call fileNamePath(Percorso, temp)

'Se devo rimuovere l'estensione
If removeExtension Then
    Dim extLen As Integer
    extLen = Len(getFileExtension(temp))
    If extLen > 0 Then temp = Left(temp, Len(temp) - extLen - 1)
End If

getFileName = temp

End Function

Public Function getFilePath(Percorso As String) As String
'Restituisce la cartella in cui  contenuto il percorso Percorso
Dim temp As String
Call fileNamePath(Percorso, , temp)
getFilePath = temp
End Function

Public Sub CreaPercorso(Percorso As String)
'Dato un percorso tipo c:\programmi\abc\def lo crea sul disco
On Error Resume Next
Dim p As Integer
p = InStr(1, Percorso, "\")
Do While p > 0              'Devo prima creare c:\ poi c:\programmi\ poi c:\programmi\abc ecc...
    MkDir Left(Percorso, p) 'Creazione della cartella
    p = InStr(p + 1, Percorso, "\")
Loop
MkDir Percorso
End Sub

Public Function getExeDllAttribute(fileName As String, wichAttribute As exeAttributes) As String
'Restituisce l'attributo voluto nel file EXE, Dll o OCX con percorso fileName

    Dim lngP As Long
    Dim lngMax As Long
    Dim intC As Long
    Dim intF As Long
    Dim strBuffer As String * 30000
    Dim cercaConZeri As String
    Dim Result As String
    Dim d As Integer
    Dim p As Integer
    
    'Scelgo cosa devo cercare
    Select Case wichAttribute
        Case ProductName
            cercaConZeri = addCharZero("ProductName") & String(3, 0)
        Case CompanyName
            cercaConZeri = addCharZero("CompanyName") & String(4, 0)
        Case FileVersion
            cercaConZeri = addCharZero("FileVersion") & String(4, 0)
        Case FileVersionMayor
            cercaConZeri = addCharZero("FileVersion") & String(4, 0)
        Case FileVersionMinor
            cercaConZeri = addCharZero("FileVersion") & String(4, 0)
        Case FileVersionRevision
            cercaConZeri = addCharZero("FileVersion") & String(4, 0)
        Case ProductVersion
            cercaConZeri = addCharZero("ProductVersion") & String(2, 0)
        Case ProductVersionMayor
            cercaConZeri = addCharZero("ProductVersion") & String(2, 0)
        Case ProductVersionMinor
            cercaConZeri = addCharZero("ProductVersion") & String(2, 0)
        Case ProductVersionRevision
            cercaConZeri = addCharZero("ProductVersion") & String(2, 0)
        Case Copyrights
            cercaConZeri = addCharZero("LegalCopyright") & String(2, 0)
        Case Description
            cercaConZeri = addCharZero("FileDescription") & String(4, 0)
        Case Comments
            cercaConZeri = addCharZero("Comments") & String(2, 0)
        Case OriginalFileName
            cercaConZeri = addCharZero("OriginalFilename") & String(2, 0)
        Case InternalName
            cercaConZeri = addCharZero("InternalName") & String(2, 0)
    End Select

    On Error GoTo err
    
    'Vedo se esiste
    If Not existFile(fileName) Then GoTo err
    
    'Devo controllare se nel file sono presenti le stringhe indicate
    'Lo faccio bufferizzandolo a pezzi. Notare che il buffer (n+1)esimo
    'contiene anche parte dell' n-esimo. Questo per trovare anche
    'propriet che sono a cavallo fra due buffers
    Dim tfreeFile As Integer
    tfreeFile = FreeFile
    Open fileName For Binary As #tfreeFile Len = 30000
    lngMax = LOF(tfreeFile) / 29000
        For lngP = 0 To lngMax
            Get tfreeFile, 29000 * lngP + 1, strBuffer
                'Cerco l'inizio del testo
                intC = InStr(1, strBuffer, cercaConZeri, 0)
                If intC > 0 Then
                    intF = InStr(intC + Len(cercaConZeri), strBuffer, Chr(0) & Chr(0))
                    If intF > 0 Then    'Se trovo la fine in questo Buffer bene, altrimenti cercher nell'altro Buffer
                        intC = intC + Len(cercaConZeri)
                        Result = removeCharZero(Mid(strBuffer, intC, intF - intC))
                        GoTo getResult
                    End If
                End If
            DoEvents
        Next
    
err:
    getExeDllAttribute = ""
    Close tfreeFile
Exit Function

getResult:
If wichAttribute = FileVersionMayor Or wichAttribute = ProductVersionMayor Then 'Se  questo cerco solo la prima parte
    d = InStr(1, Result, ".")
    getExeDllAttribute = Left(Result, d - 1)
ElseIf wichAttribute = FileVersionMinor Or wichAttribute = ProductVersionMinor Then 'Se  questo cerco solo la seconda parte
    d = InStr(1, Result, ".")
    p = InStr(d + 1, Result, ".")
    If p > 0 Then   'Se c' anche il numero della revisione
        getExeDllAttribute = Mid(Result, d + 1, p - d - 1)
    Else            'Altrimenti...
        getExeDllAttribute = Right(Result, Len(Result) - d)
    End If
ElseIf wichAttribute = FileVersionRevision Or wichAttribute = ProductVersionRevision Then 'Se  questo cerco solo la seconda parte
    d = InStr(1, Result, ".")
    p = InStr(d + 1, Result, ".")
    If p = 0 Then   'Non  indicata la revisione
        getExeDllAttribute = ""
    Else            'E' indicata la revisione
        getExeDllAttribute = Right(Result, Len(Result) - p)
    End If
Else    'Altrimenti  gi pronto
    getExeDllAttribute = Result
End If

Close tfreeFile
End Function


Public Function existFile(fileName As String, Optional UseDir As Boolean = True) As Boolean
'Restituisce True se il file fileName esiste
'Inizializzo
existFile = False

If UseDir Then
    If Dir(fileName) <> "" And fileName <> "" Then If Not isFileAttribute(fileName, DirectoryAtt) Then existFile = True
Else
    On Error GoTo err
    Dim tfreeFile As Integer
    tfreeFile = FreeFile
    Open fileName For Input As #tfreeFile
    Close #tfreeFile
    existFile = True
    Exit Function
err:
    err.Clear
    existFile = False
    Close #tfreeFile
End If

End Function

Public Function existFolder(folderName As String) As Boolean
'Restituisce True se la cartella folderName esiste
'Inizializzo
existFolder = False

If Dir(folderName, 30) <> "" Then If isFileAttribute(folderName, DirectoryAtt) Then existFolder = True
End Function

Public Function getDiskName(diskName As String) As String
Dim lpRootPathName As String   ''parametri per chiamata API
Dim lpVolumeNameBuffer As String
Dim nVolumeNameSize As Long
Dim lpVolumeSerialNumber As Long
Dim lpFileSystemNameBuffer As String
Dim nFileSystemNameSize As Long
Dim lpMaximumComponentLength As Long
Dim lpFileSystemFlags As Long

lpRootPathName = diskName
lpVolumeNameBuffer = String(256, 0)
lpFileSystemNameBuffer = String(256, 0)
'lpMaximumComponentLength = 0
'lpVolumeSerialNumber = 0
'lpFileSystemFlags = 0

Dim LReturnCode As Long

If GetVolumeInformation(lpRootPathName, lpVolumeNameBuffer, Len(lpVolumeNameBuffer), lpVolumeSerialNumber, lpMaximumComponentLength, lpFileSystemFlags, lpFileSystemNameBuffer, Len(lpFileSystemNameBuffer)) = 0 Then
    getDiskName = ""
Else
    'Ok
    getDiskName = Left(lpVolumeNameBuffer, InStr(1, lpVolumeNameBuffer, Chr(0)) - 1)
End If

End Function

Public Function RimuoviFile(ByRef NomeFile As String) As Boolean
On Error GoTo ErrKilling
Kill NomeFile
RimuoviFile = True
Exit Function
ErrKilling:
err.Clear
RimuoviFile = False
End Function

Public Sub PulisciCartella(ByVal Cartella As String, Optional RemoveAlsoThis As Boolean = False, Optional RecurseFolders As Boolean = False)
'Cancella Tutti i files contenuti in una cartella
'Eventualmente cancella anche tutte le sottocartelle
'Eventualmente cancella anche se stessa
On Error Resume Next
If Right(Cartella, 1) <> "\" Then Cartella = Cartella & "\"
'Cancello tutti i files
Kill Cartella & "*.*"
err.Clear

Dim d As Integer

If RecurseFolders Then
    Dim newDir As String
    Dim recurseDir() As String
    Dim tempStr As String
    ReDim recurseDir(-1 To -1)
    
    newDir = Dir$(Cartella & "*.*", vbDirectory)
    Do While newDir <> ""         'Continuo fino a che Dir$ mi passa una cartella
        If newDir <> "." And newDir <> ".." Then    'A meno che non mi passi . o ..
            If (GetAttr(Cartella & newDir) And vbDirectory) = vbDirectory Then ' una cartella
                Call AddToArray(recurseDir, newDir)
            Else
                Stop
            End If
        End If
        newDir = Dir$         'Richiedo un'altra cartella
    Loop
    For d = 0 To UBound(recurseDir) 'Ora elimino le sottocartelle
        Call PulisciCartella(Cartella & recurseDir(d) & "\", True, True)
    Next d
End If

If RemoveAlsoThis Then RmDir Left$(Cartella, Len(Cartella) - 1)

err.Clear
End Sub

Public Function getExeDependants(fileName As String, depFiles() As String) As Boolean
'Restituisce False se ha riscontrato errori
'Aggiunge a depFiles (Array dinamico) tutti files dai quali dipende
'il file eseguibile.
'N.B. Il metodo NON  infallibile!
    
Dim lngP As Long
Dim lngMax As Long
Dim strBuffer As String * 30000   'Una stringa usata come Buffer
Dim d As Integer: Dim b As Integer

On Error GoTo err

'Vedo se esiste il file
If Not existFile(fileName) Then GoTo err

d = UBound(depFiles) 'Lo uso dopo per la ricorsione

Dim tfreeFile As Integer
tfreeFile = FreeFile
Open fileName For Binary As tfreeFile Len = 30000  'Len 30000 indica la lunghezza del buffer
    lngMax = LOF(tfreeFile) / 29000         'Leggo ogni 29000 byte con buffer da 30000 per avere un margine intermedio
    For lngP = 0 To lngMax
        Get tfreeFile, lngP * 29000 + 1, strBuffer
        SearchDepInBuffer strBuffer, depFiles 'Cerca le dipendenze e le aggiunge alla lista in resULts
        DoEvents
    Next
Close

getExeDependants = True
Exit Function
err:
getExeDependants = False
Close
End Function

Public Function searchForDepandantsPaths(ByVal defaultPath As String, ByVal SistemPath As String, depFiles() As String, Optional leftPosInArray As Integer = 0) As Boolean
'Questa funzione modifica l'array depFiles con i percorsi completi
'dei files al posto dei semplici nomi. Ovviamente se li trova!
'La ricerca viene effettuata nella defaultPath oppure nella cartella
'di sistema
If Right(defaultPath, 1) <> "\" Then defaultPath = defaultPath & "\"
If Right(SistemPath, 1) <> "\" Then SistemPath = SistemPath & "\"
Dim d As Integer

searchForDepandantsPaths = True
For d = leftPosInArray To UBound(depFiles)
    If Not existFile(defaultPath & depFiles(d)) Then
        If Not existFile(SistemPath & depFiles(d)) Then
            searchForDepandantsPaths = False
        Else
            depFiles(d) = SistemPath & depFiles(d)
        End If
    Else
        depFiles(d) = defaultPath & depFiles(d)
    End If
Next d
End Function

'**************************************************************
'***********    Procedure e Funzioni Private    ***************
'***********   Utilizzate da quelle Pubbliche   ***************
'**************************************************************
Private Function Findfile(strFileName As String) As WIN32_FIND_DATA
'Cerca i dati di un file
Dim Win32Data As WIN32_FIND_DATA
Dim plngFirstFileHwnd As Long
Dim plngRtn As Long

plngFirstFileHwnd = FindFirstFile(strFileName, Win32Data)   'Ottengo le informazioni dal file
If plngFirstFileHwnd < 1 Then
  Findfile.cFileName = "Errore"                             'Errore
Else
  Findfile = Win32Data
End If
plngRtn = FindClose(plngFirstFileHwnd)  'Chiudo il file
End Function

Public Function AddToArray(strArray() As String, elemento As String, Optional OnlyIfNotAlreadyPresent As Boolean = False) As Boolean
'Aggiunge un elemento all'array di stringhe
'Restituisce false solo se:
'    stato richiesto di aggiungere solo se non c' e c'era
On Error Resume Next
Dim nextPos As Integer
Dim leftPos As Integer

'Controllo se c' gi... se richiesto
If OnlyIfNotAlreadyPresent Then
    If CheckStrInStrArray(elemento, strArray) Then AddToArray = False: Exit Function
End If
nextPos = UBound(strArray) + 1
leftPos = LBound(strArray)
ReDim Preserve strArray(leftPos To nextPos)
strArray(nextPos) = elemento
AddToArray = True
End Function

Private Function AddToArrayInt(intArray() As Integer, elemento As Integer, Optional OnlyIfNotAlreadyPresent As Boolean = False) As Boolean
'Aggiunge un elemento all'array di Integer
'Restituisce false solo se:
'    stato richiesto di aggiungere solo se non c' e c'era
On Error Resume Next
Dim nextPos As Integer
Dim leftPos As Integer

'Controllo se c' gi... se richiesto
If OnlyIfNotAlreadyPresent Then
    If CheckIntInIntArray(elemento, intArray) Then AddToArrayInt = False: Exit Function
End If
nextPos = UBound(intArray) + 1
leftPos = LBound(intArray)
ReDim Preserve intArray(leftPos To nextPos)
intArray(nextPos) = elemento
AddToArrayInt = True
End Function

Private Sub SearchDepInBuffer(Buffer As String, resULts() As String)
'Ricerca una dipendenza nel Buffer e la salva nell'array

    Dim intC As Integer             'Un contatore
    Dim daAggiungere As String      'Un file da aggiungere
    Dim CaratteriAmmessi As String  'Per assicurarsi che sia una parola con caratteri possibili in un nome di file
    Dim strC As String              'Ogni carattere letto
    Dim intX As Integer             'Contatore della lunghezza della parola
    Dim findString As String
    
    CaratteriAmmessi = "_ABCDEFGHIJKLMNOPQRSTUVWXYZ.-0123456789"
    
    findString = ".DLL"         'Ricerco le dll
    GoSub cercaFindString
    
    findString = ".OCX"         'Ricerco gli Ocx
    GoSub cercaFindString
    
    'Potrei aggiungere qualcos'altro da cercare se ne ho bisogno...
    Exit Sub
    
cercaFindString:    'Ricerca di findString
    
    intC = InStr(UCase(Buffer), findString)
    Do While intC > 0   'Fino a quando trovo .dll
        intX = 0
        strC = Mid(Buffer, intC - intX, 1)   'Un carattere
        Do Until (InStr(CaratteriAmmessi, UCase(strC)) = 0) 'Fino a che trovo caratteri ammessi in un nome di file
            intX = intX + 1                     'Incremento il contatore
            If intC - intX < 1 Then GoTo prossimaRicerca 'Sicuramente avr trovato questo nome nel Buffer precedente
            strC = Mid(Buffer, intC - intX, 1)  'Un carattere
        Loop
        If intX < 2 Then GoTo prossimaRicerca
        'Se sono qui ho trovato un file .dll o .ocx e lo aggiungo all'array
        daAggiungere = Mid(Buffer, intC - (intX - 1), intX + Len(findString) - 1)
        'Lo aggiungo a meno che non sia gi presente
        If Not CheckStrInStrArray(daAggiungere, resULts) Then
            AddToArray resULts, daAggiungere
        End If
prossimaRicerca:
        'Cerco la prossima presenza nel Buffer
        intC = InStr(intC + 1, UCase(Buffer), findString)
    Loop

Return 'Torno su

End Sub

Public Function CheckStrInStrArray(ByVal Check As String, strArray() As String, Optional ByRef foundInPosition As Integer) As Boolean
'Restituisce TRUE se una stringa  gi inclusa in un array di stringhe
Dim myCheck As String
Dim t As Integer

myCheck = LCase(Check)
For t = LBound(strArray) To UBound(strArray)
    If LCase(strArray(t)) = myCheck Then
        CheckStrInStrArray = True
        foundInPosition = t
        Exit Function
    End If
Next t
CheckStrInStrArray = False
End Function

Public Function CheckIntInIntArray(ByVal Check As Integer, intArray() As Integer, Optional ByRef foundInPosition As Integer) As Boolean
'Restituisce TRUE se un Integer  gi incluso in un array di Integer
Dim t As Integer
For t = LBound(intArray) To UBound(intArray)
    If intArray(t) = Check Then
        CheckIntInIntArray = True
        foundInPosition = t
        Exit Function
    End If
Next t
CheckIntInIntArray = False
End Function

Private Function addCharZero(stringa As String) As String
'Aggiunge un carattere 0 (ASCII) dopo ogni carattere della parola stringa
'Serve per gli attributi in un file Eseguibile o Dll
Dim i As Integer
Dim temp As String

For i = 1 To Len(stringa)
    temp = temp & Mid(stringa, i, 1) & Chr(0)
Next i
addCharZero = temp
End Function

Private Function removeCharZero(stringa As String) As String
'Toglie tutti i caratteri 0 (ASCII) dalla stringa
'Serve per gli attributi in un file Eseguibile o Dll
Dim temp As String
Dim i As Integer
Dim tChar As String * 1

temp = ""

For i = 1 To Len(stringa)
    tChar = Mid(stringa, i, 1)
    If tChar <> Chr(0) Then temp = temp & tChar
Next i
removeCharZero = temp
End Function

Private Function imponiDueCifre(ByVal cifra As Integer) As String
If cifra < 10 Then imponiDueCifre = "0" & CStr(cifra) Else imponiDueCifre = CStr(cifra)
End Function
