Attribute VB_Name = "WinIniApi"
'*********************************************************************
'Titolo: Ini Files Editor
'Autore: Giorgio Bernardi
'E-Mail: BGSoftware@hotmail.com
'Data  : Novembre 2000
'Descrizione: Modulo per utilizzare i files *.ini (o *.inf)
'Info:  Potete usare e modificare a piacimento questo codice
'*********************************************************************

'Dichiarazione delle Api necessarie
Private Declare Function GetPrivateProfileString Lib "kernel32" Alias "GetPrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long
'Private Declare Function GetPrivateProfileSection Lib "kernel32" Alias "GetPrivateProfileSectionA" (ByVal lpAppName As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long
Private Declare Function WritePrivateProfileString Lib "kernel32" Alias "WritePrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpString As Any, ByVal lpFileName As String) As Long
Private Declare Function WritePrivateProfileSection Lib "kernel32" Alias "WritePrivateProfileSectionA" (ByVal lpAppName As String, ByVal lpString As String, ByVal lpFileName As String) As Long

'Chiave di un file Ini
'Public Type IniKeyType
'    keyName As String
'    KeyValue As String
'End Type

'Intera Sezione di un file Ini
'Public Type IniSectionType
'    Name As String
'    Keys() As IniKeyType
'End Type

'Intero File Ini
'Public Type IniFileType
'    Name As String
'    Sezioni() As IniSectionType
'End Type

'**************************************************************************
'FUNZIONI PER LA LETTURA DI INFORAMAZIONI DIRETTAMENTE DAL FILE
'**************************************************************************
'
'   Queste funzioni, restituiscono il valore letto da un file.
'   Tutte le funzioni hanno la medesima firma:
'       fileName => Indica il percorso compelto del file da cui leggere
'       Section =>  Indica la sezione in cui si trova la chiave
'       keyName =>  Indica il nome della chiave
'       ok (Opzionale) => Viene impostato a True se il valore  stato letto correttamente, a False nel caso opposto
'       defaultString (Opz) => Indica il valore di default da restituire in caso di errore in lettura (nei diversi casi si tratter di una stringa, un intero ecc...)
'       bufferSize (Opz)    => Indica la lunghezza massima da leggere per il dato. Un intero, per esempio, non pu essere pi lungo di 6 caratteri (5 + il segno)
'
'   Le funzioni utilizzano la getIniString come base ed effettuano la conversione nel tipo appropriato.
'
'   LISTA DELLE FUNZIONI
'       getIniString => Restituisce una stringa
'       getIniBool  =>  Restituisce un valore Booleano
'       getIniInt   =>  Restituisce un valore intero Integer
'       getIniCurr  =>  Restituisce un valore di tipo Currency
'       getIniLong  =>  Restituisce un valore di tipo Long

Public Function getIniString(ByVal fileName As String, ByVal Section As String, ByVal keyName As String, Optional ok As Boolean = True, Optional ByVal defaultString As String = "", Optional bufferSize As Long = 32000) As String
Dim returnString As String
Dim myDefault As String
Dim StrLen As Long
Dim res As Long

'Inizializzo le variabili
ok = True
If defaultString = "" Then myDefault = Space(32000)
returnString = Space(bufferSize)

'Richiedo il dato
res = GetPrivateProfileString(Section, keyName, myDefault, returnString, bufferSize, fileName)

'Vedo se la risposta  un errore
If res = 0 Then
    getIniString = defaultString
    ok = False
    Exit Function
End If

'Cerco il carattere chr(0) che chiude la stringa
StrLen = InStr(1, returnString, Chr(0), vbBinaryCompare)

If StrLen <> 0 Then getIniString = Left(returnString, StrLen - 1) Else getIniString = returnString

End Function


'**************************************************************************
'FUNZIONI PER LA SCRITTURA DI DATI
'**************************************************************************
'
'   Le funzioni di scrittura richiedono di indicare la posizione di scrittura e i dati da inserire
'   La variabile addLen, viene incrementata con lo spazio occupato nel file dall'inserimento.
'
Public Function setIniVal(ByVal fileName As String, ByVal Section As String, ByVal keyName As String, ByVal Value As Variant, Optional ByRef addlen As Long) As Boolean
'Utilizzata per ogni tipo di dato. I Boolean vengono automaticamente convertiti
'in True o False in modo da essere letti facilmente con le funzioni sopra indicate
'Restituisce True se la scrittura  andata a buon fine
    setIniVal = CBool(WritePrivateProfileString(Section, keyName, CStr(Value), fileName))
    addlen = addlen + Len(keyName & CStr(Value)) + 3 '1 per = , 2 per ritorno a capo
End Function

Public Function setIniSection(fileName As String, NomeSezione As String, Sezione As String, Optional ByRef addingLen As Long) As Boolean
'Scrive una intera sezione su un file
'Restituisce True se la scrittura  andata a buon fine
'ATTENZIONE: La funzione cancella totalmente i dati presenti in precedenza in quella sezione
    setIniSection = CBool(WritePrivateProfileSection(NomeSezione, Sezione, fileName))
    addingLen = addingLen + Len(Sezione) + Len(NomeSezione) + 4
End Function

